from textual.app import App
from textual.widgets import DataTable, Header, Input
from textual.containers import Container
from datetime import datetime

from .controller import get_tasks, add_task, update_task, TaskStatus
from .db import initialize_db

column_to_field = {
    1: "text",
    2: "status",
    3: "type",
    4: "due",
    5: "category_name",
}


def get_colored_status(status: str) -> str:
    colors = {
        "zero": "#666666",
        "wip": "#33aa99",
        "blocked": "#cc9900",
        "done": "#009900",
    }
    return f"[{colors[status]}]{status}[/]"


def get_colored_category(category: str) -> str:
    hash_val = sum(ord(c) for c in category)
    hue = hash_val % 360
    color = f"rgb({hue},200,200) on default"
    return f"[{color}]{category}[/]"


class TaskManagerApp(App):
    CSS = """
    Input {
        dock: bottom;
        margin: 0;
        padding: 0;
        border: none;
        background: $boost;
    }
    """

    BINDINGS = [
        # movement
        ("h", "cursor_left", "Left"),
        ("j", "cursor_down", "Down"),
        ("k", "cursor_up", "Up"),
        ("l", "cursor_right", "Right"),
        ("g", "cursor_top", "Top"),
        ("G", "cursor_bottom", "Bottom"),
        # filtering & editing
        ("/", "start_search", "Search"),
        ("c", "start_edit", "Edit Cell"),
        ("escape", "cancel_edit", "Cancel Edit"),
        # other
        ("a", "add_task", "Add Task"),
        ("q", "quit", "Quit"),
    ]

    def __init__(self):
        super().__init__()
        self.search_query = ""
        self.edit_row = None
        self.edit_column = None

    def compose(self):
        self.header = Header()
        self.table = DataTable()
        self.input_bar = Input(id="input_bar")
        self.input_bar.display = False
        yield self.header
        yield Container(self.table)
        yield self.input_bar

    def on_mount(self):
        self.table.add_columns("ID", "Task", "Status", "Type", "Due", "Category")
        self.refresh_tasks()

    def action_cursor_left(self):
        self.table.move_cursor(column=self.table.cursor_column - 1)

    def action_cursor_right(self):
        self.table.move_cursor(column=self.table.cursor_column + 1)

    def action_cursor_up(self):
        self.table.move_cursor(row=self.table.cursor_row - 1)

    def action_cursor_down(self):
        self.table.move_cursor(row=self.table.cursor_row + 1)

    def action_cursor_top(self):
        self.table.move_cursor(row=0)

    def action_cursor_bottom(self):
        self.table.move_cursor(row=self.table.row_count - 1)

    def refresh_tasks(self):
        self.table.clear()

        tasks = get_tasks(self.search_query)
        for task in tasks:
            due_str = task.due.strftime("%Y-%m-%d") if task.due else " - "
            category = get_colored_category(
                task.category.name if task.category else " - "
            )
            status = get_colored_status(task.status)

            self.table.add_row(
                str(task.id),
                task.text,
                status,
                task.type,
                due_str,
                category,
                key=str(task.id),
            )

    def action_add_task(self):
        """Add a new task with default values."""
        add_task(
            text="New Task",
            type="task",
            status=TaskStatus.ZERO.value,
            category="main",
        )
        self.refresh_tasks()

    # Control of edit bar ####################

    def _show_input(self, mode, start_value):
        self.mode = mode
        self.input_bar.display = True
        self.input_bar.value = start_value
        self.set_focus(self.input_bar)

    def _hide_input(self):
        self.input_bar.display = False
        self.set_focus(self.table)

    def action_cancel_edit(self):
        self._hide_input()

    def action_start_search(self):
        self._show_input("search", "")

    def action_start_edit(self):
        if self.table.cursor_row is None or self.table.cursor_column == 0:
            return

        self.saved_cursor_pos = (self.table.cursor_row, self.table.cursor_column)
        current_value = self.table.get_cell_at(
            (self.table.cursor_row, self.table.cursor_column)
        )
        # TODO: remove color codes
        self._show_input("edit", current_value)

    def on_input_submitted(self, event: Input.Submitted):
        if self.mode == "search":
            self.search_query = event.value
            self.refresh_tasks()
        elif self.mode == "edit":
            self.apply_change(event.value)
        self._hide_input()

    def apply_change(self, new_value):
        row, col = self.saved_cursor_pos
        task_id = int(self.table.get_cell_at((row, 0)))

        field = column_to_field[col]
        update_data = {}

        if field == "due":
            try:
                update_data["due"] = datetime.strptime(new_value, "%Y-%m-%d")
            except ValueError:
                self.notify("Invalid date format. Use YYYY-MM-DD")
        elif field == "status":
            try:
                TaskStatus(new_value)  # validate status
                update_data["status"] = new_value
            except ValueError:
                self.notify(f"Invalid status. Use: {[s.value for s in TaskStatus]}")
        else:
            update_data[field] = new_value

        try:
            update_task(task_id, **update_data)
            self.refresh_tasks()
        except Exception as e:
            self.notify(f"Error updating task: {str(e)}")
        finally:
            self.action_cancel_edit()


def run():
    initialize_db()
    app = TaskManagerApp()
    app.run()


if __name__ == "__main__":
    run()
